/*
 * Copyright (c) 2000, 2001 Murray Stokely
 * Copyright (c) 2000, 2001 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Driver for National Instruments PCI E-Series data acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, May 1995 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 */

#define NI_VENDOR_ID         0x1093

#define DEV_AI    0
#define DEV_DIO   1
#define DEV_GPCT0 2
#define DEV_GPCT1 3
#define DEV_STATS 4

#define AI_BUSY      0x0001
#define DIO_BUSY     0x0002
#define GPCT0_BUSY   0x0004
#define GPCT1_BUSY   0x0008

struct ni_softc {
    struct mtx lock;
    struct resource *bar0res;	/* BAR0 points to the base address of
				   the MITE registers */
    int bar0id;
    bus_space_tag_t bar0_bt;
    bus_space_handle_t bar0_bh;
    struct resource *bar1res;	/* BAR1 points to the base address of
                                   the board registers such as the
                                   DAQ-STC */
    int bar1id;
    bus_space_tag_t bar1_bt;
    bus_space_handle_t bar1_bh;

    struct resource *irqres;
    int irqid;
    bus_space_tag_t irq_bt;
    bus_space_handle_t irq_bh;
    void *handler;

    int unit;
    struct daq_device *device_info;
    
    int use;

    /* Digital I/O stuff */
    int dio_flags;
    int wrmask;
    int dio;

    /* Analog input stuff */
    uint32_t scan_count;		/* 24bit scan counter  */
    uint32_t scan_interval;		/* 24bit scan interval counter */
    uint16_t sample_interval;	/* 16bit sample interval counter */
    int ai_chancnt;
    uint32_t *ai_channels;

    /* We need a memory buffer here for the FIFO data */
    int status;

    /* Device nodes from make_dev */
    dev_t daqdev;
    dev_t diodev;
    dev_t cntdev0;
    dev_t cntdev1;
    dev_t statdev;
    uint16_t *fifo;
    uint16_t fifocnt;
    uint16_t startcnt;
    int reg[256];
};

/* Prototypes */

void DAQ_STC_Windowed_Mode_Write1(struct ni_softc *sc, uint16_t address, uint8_t value);
void DAQ_STC_Windowed_Mode_Write(struct ni_softc *sc, uint16_t address, uint16_t value);
void DAQ_STC_Windowed_Mode_Masked_Write(struct ni_softc *sc, uint16_t address, uint16_t value, uint16_t mask);
void DAQ_STC_Windowed_Mode_Strobe_Write(struct ni_softc *sc, uint16_t address, uint16_t value);
uint8_t DAQ_STC_Windowed_Mode_Read1(struct ni_softc *sc, uint16_t address);
uint16_t DAQ_STC_Windowed_Mode_Read(struct ni_softc *sc, uint16_t address);
void board_write(struct ni_softc *sc, uint16_t address, uint16_t value);
void board_write1(struct ni_softc *sc, uint16_t address, uint8_t value);
uint16_t board_read(struct ni_softc *sc, uint16_t address);
uint16_t board_read1(struct ni_softc *sc, uint8_t address);
void AI_Reset_All(struct ni_softc *sc);
void AI_Ack_All(struct ni_softc *sc);
void AI_Interrupt_Enable(struct ni_softc *sc);
void Configure_Board(struct ni_softc *sc, uint32_t channel);
void MSC_Clock_Configure(struct ni_softc *sc);
void MSC_IRQ_Configure(struct ni_softc *sc);
void Clear_FIFO(struct ni_softc *sc);
void Clear_Configuration_Memory(struct ni_softc *sc);
void AI_Board_Personalize(struct ni_softc *sc);
void AI_Initialize_Configuration_Memory_Output(struct ni_softc *sc);
void AI_Board_Environmentalize(struct ni_softc *sc);
void AI_Trigger_Signals(struct ni_softc *sc);
void Number_of_Scans(struct ni_softc *sc);
void AI_Scan_Start(struct ni_softc *sc);
void AI_End_of_Scan(struct ni_softc *sc);
void AI_Start_The_Acquisition(struct ni_softc *sc);
int dio_ioctl(struct ni_softc *sc, u_long cmd, caddr_t data, int fflag, struct proc *p);
int ai_ioctl(struct ni_softc *sc, u_long cmd, caddr_t data, int fflag, struct proc *p);
void AI_Add_Channel(struct ni_softc *sc, uint32_t arg);
void AI_Arming(struct ni_softc *sc);
void Convert_Signal(struct ni_softc *sc);
void ni_handler(void *xsc);
uint32_t Encode_Channel(struct ai_channel *chan);

int gpct0_ioctl(struct ni_softc *sc, u_long cmd, caddr_t data, int fflag, struct proc *p);
void G0_Reset_All(struct ni_softc *sc);
void G0_Arm_All(struct ni_softc *sc);
void G0_Simple_Event_Counting(struct ni_softc *sc);
void G0_Buffered_Event_Counting(struct ni_softc *sc);
void G0_Single_Pulse_Generation(struct ni_softc *sc);
