/*
 * Copyright (c) 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Test program for `ni' device driver for National Instruments data
 * acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 * Murray Stokely
 * 13 Mar 2001
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <string.h>

#ifdef __FreeBSD__
#include "niio.h"
#define DIO_DEVICE "/dev/nidio0"
#elif defined __sun
#include "niio_solaris.h"
#define DIO_DEVICE "/devices/pci@1f,0/pci@1/pci1093,2a60@3:nidio"
#else
#error "Unsupported Platform!"
#endif

int test_ai();
void Example_One();
void Example_Two();
int test_dio();
int test_gpct();
int simpletest(int fd);

float
toCelsius(float voltage) {
    return (voltage * 100);
}

float
toFarenheit(float voltage) {
    return ((voltage * 100) * 9/5) + 32;
}


int 
main(int argc, char **argv) {
    char ch; /* This must be signed for getopt comparison to EOF to work */
    int tests = 0;

    /* Should we check all the devices or just one? */

    printf("NI Test Application\n");
    /* First thing to do is to retrieve information about device. */


    while ((ch = getopt(argc, argv, "adgh")) != EOF) {
	switch(ch) {
	case 'd':
	    if (test_dio())
		return -1;
	    tests++;
	    break;
	case 'a':
	    if (test_ai())
		return -1;
	    tests++;
	    break;
	case 'g':
	    if (test_gpct())
		return -1;
	    tests++;
	    break;
	default:
	    printf("Not supported! (%d) (%d)\n", ch, EOF);
	    goto done;
	}
    }
    if (!tests) {
	printf("Usage : test -d\n");
    }
done:
    return 0;
}

int
simpletest(int fd) {
    unsigned char out=0, in=0;
    
    if (write(fd, &out, 1) < 1) {
	perror("Can't write to nidio0");
	return -1;
    }
    if (read(fd, &in, 1) < 1) {
	perror("Can't read from nidio0");
	return -1;
    }
    printf("0x%2x - 0x%2x\n", out, in);
    sleep(1);

    out = 0xFF;
    if (write(fd, &out, 1) < 1) {
	perror("Can't write to nidio0");
	return -1;
    }
    if (read(fd, &in, 1) < 1) {
	perror("Can't read from nidio0");
	return -1;
    }
    printf("0x%2x - 0x%2x\n", out, in);
    sleep(1);
}

int
test_dio() {
    int fd, i;
#ifdef __FreeBSD__
    int wrmask;
#endif

    printf("Testing Digital I/O (" DIO_DEVICE ")\n"
	   "<value written> - <value read>\n");
 
    fd = open(DIO_DEVICE, O_RDWR);
    if (fd < 0) {
	printf("Couldn't open!\n");
	return -1;
    }

    wrmask = 0x00;
#ifdef __FreeBSD__
    if (ioctl(fd, NICTL_DIO_WRMASK, &wrmask)) {
#elif defined __sun
    if (ioctl(fd, NICTL_DIO_WRMASK, wrmask)) {
#endif
	perror("Could not set write mask!");
	return -1;
    }
    printf("Writemask : 0x00\n");
    /* The read/write mask is 0x0F (bits 8..5 = read * bits 4..0 write) */
    if (simpletest(fd))
	return -1;

    wrmask = 0x0F;
#ifdef __FreeBSD__
    if (ioctl(fd, NICTL_DIO_WRMASK, &wrmask)) {
#elif defined __sun
    if (ioctl(fd, NICTL_DIO_WRMASK, wrmask)) {
#endif
	printf("Could not set write mask!\n");
	return -1;
    }
    printf("Writemask : 0x0F\n");
    if (simpletest(fd))
	return -1;

    wrmask = 0xFF;
#ifdef __FreeBSD__
    if (ioctl(fd, NICTL_DIO_WRMASK, &wrmask)) {
#elif defined __sun
    if (ioctl(fd, NICTL_DIO_WRMASK, wrmask)) {
#endif
	printf("Could not set write mask!\n");
	return -1;
    }
    printf("Writemask : 0xFF\n");
    if (simpletest(fd))
	return -1;

    close(fd);
    return 0;
}

int
test_ai() {
    printf("\nTesting Analog Input (/dev/nidaq0)\n");

    Example_One();
    Example_Two();
    return 0;
}

int
test_gpct() {
    int fd;
    uint32_t count;
    struct gpct_event event;

    printf("\nTesting GPCT0 (/dev/nigpct00)\n");

    if ((fd = open("/dev/nigpct00", O_RDONLY)) < 0) {
	printf("Couldn't open!\n");
	return -1;
    }
    if (ioctl(fd, NICTL_GPCT_RESET, &fd)) printf("Could not reset counter!\n");

    printf("Reading channel 0 in RSE\n");
    bzero(&event, sizeof(event));
    event.type = SIMPLE_EVENT_COUNTING;
    event.source   = 1;
    event.polarity = 1;
    event.gate     = 0;
    event.output_polarity = 9;
    event.gate_polarity   = 0;
    event.output_mode     = 0;
    event.up_down         = 0;

    if (ioctl(fd, NICTL_GPCT_SETUP, &event)) printf("Could not setup counter!\n");
    if (ioctl(fd, NICTL_GPCT_ARM, &fd)) printf("Could not arm counter!\n");
    printf("Counter has been armed, sleeping 3!\n");
    sleep(3);
    if (read(fd, &count, sizeof(count)) < 1) {
	perror("Read!");
    } else {
	printf("Counter value : %d\n", count);
    }
    sleep(3);
    if (read(fd, &count, sizeof(count)) < 1) {
	perror("Read!");
    } else {
	printf("Counter value : %d\n", count);
    }
    if (ioctl(fd, NICTL_GPCT_RESET, &fd)) printf("Could not reset counter!\n");
    close(fd);
}

/* 
 * From [2] 4-9
 * Acquires one sample from channel 0 
 */

void
Example_One() {
    int fd;
    int16_t in;
    float voltage;
    struct ai_channel chan;
    struct daq_device device_info;

    printf("-------------------------------------------------------\n"
	   " Example One (Analog Input - One sample)\n"
	   "-------------------------------------------------------\n\n");

    if ((fd = open("/dev/nidaq0",O_RDONLY)) < 0) {
	printf("Couldn't open!\n");
	return;
    }

    if (ioctl(fd, NICTL_DEVICE_INFO, &device_info)) {
	perror("Could not get device info!\n");
    } else {
    printf("%s\n"
	   "%d analog input channels with %d-bit resolution.\n\n",
	   (char *)device_info.description, device_info.ai_channels, device_info.ai_resolution);
    }

    printf("Reading channel 0 in RSE\n");
    bzero(&chan, sizeof(chan));
    chan.num  = 0;
    chan.mode = RSE;
    chan.gain = 1;

    if (ioctl(fd, NICTL_AI_CONVERT, &chan)) printf("Could not start conversion!\n");

    sleep(1);

    if ((read(fd, &in, sizeof(in)))< sizeof(in)) {
	perror("cant read nidaq"); 
	exit(2);
    }
    printf("We read in : %d\n", in);
    voltage = ((float)in * 10.0) / 4096.0; /* Get capabilities here? */
    printf("%5.2fV\t%5.2fdeg C\t%5.2fdeg F\n", voltage, 
	   toCelsius(voltage), toFarenheit(voltage));
    close(fd);
    return;
}

/* 
 * From [2] 4-9
 * Acquires one sample from channel 0 
 */

void
Example_Two() {
    int fd;
    int16_t in[10];
    int i;
    float voltage;
    struct ai_scaninfo scan;
    struct ai_channel chan;

    printf("-------------------------------------------------------\n"
	   " Example Two (Analog Input - Samples)\n"
	   "-------------------------------------------------------\n\n");

    if ((fd = open("/dev/nidaq0",O_RDONLY)) < 0) {
	printf("Couldn't open!\n");
	return;
    }


    printf("Setting up channel 0 in RSE\n");
    bzero(&chan, sizeof(chan));
    chan.num  = 0;
    chan.mode = RSE;
    chan.gain = 1;
    if (ioctl(fd, NICTL_AI_ADDCH, &chan)) printf ("Could not add channel!\n");

    bzero(&scan, sizeof(scan));
    scan.scans = 10;
    scan.scan_interval = 12500;
    scan.sample_interval = 0;/*50;*/

    if (ioctl(fd, NICTL_AI_SCAN_SETUP, &scan)) {
        printf("Could not setup scan!\n");
        exit(2);
    }
    if (ioctl(fd, NICTL_AI_SCAN_START, &scan)) {
        printf("Could not start scan!\n");
        exit(2);
    }

    sleep(1);

    if ((read(fd, &in, sizeof(in)))< sizeof(in)) {
	perror("cant read nidaq"); 
	exit(2);
    }
    for (i=0;i<10;i++) {
	printf("We read in : %d\n", in[i]);	
	voltage = ((float)in[i] * 10) / 4096;
	printf("%5.2fV\t%5.2fdeg C\t%5.2fdeg F\n", voltage, 
	       toCelsius(voltage), toFarenheit(voltage));
    }
    close(fd);
    return;
}
